"""
how to convert a caleydo project (without computation)
start hacked version and load the project
within org.caleydo.data the csv files will generated
rename the heterogeneous column csv file: xxx_cols.csv to xxx_desc.csv and edit it
1. remove header
2. fix name
3. add additional columns for the type: string (extra column the max length), int8, int16,int32, float16, float32, float64, enum (add as extra columns the categories)
convert it
"""

import tables
import numpy as np
import glob
import json
import os

__author__ = 'Samuel Gratzl'


def convert_it(base):
  h5 = tables.open_file(base + '.h5', 'w')

  def clean_name(name):
    n = name.lower().replace(' ', '').replace('-', '')
    n = n.replace('$', '').replace('(', '_').replace('.', '_')
    n = n.replace(')', '').split('/')[-1]

    if n[0].isdigit():
      n = '_' + n
    return n

  for f in glob.glob(base + '/*_data.csv'):
    name = f.replace('_data.csv', '')
    cleaned = clean_name(name)
    print(cleaned)

    group = h5.create_group('/', cleaned, name.split('/')[-1])

    def load_stratification(ids, idtype, origin):
      if not os.path.exists(name + '_' + idtype + '.json'):
        return None
      last = None, None
      with open(name + '_' + idtype + '.json') as fs:
        strats = json.load(fs)
        for key, value in strats.items():

          s = h5.create_group('/', clean_name(cleaned + '_' + key), origin + '/' + key)
          h5.set_node_attr(s, 'type', 'stratification')
          h5.set_node_attr(s, 'idtype', idtype)
          h5.set_node_attr(s, 'origin', origin)
          last = [], key
          for gg, indices in value.items():
            last[0].extend(indices)
            h5.create_array(s, clean_name(gg), ids[indices], gg)
      return last

    with open(name + '_rows.csv', 'r') as cc:
      line = cc.readline().split(';')
      rowtype = line[1].strip()
      h5.set_node_attr(group, 'rowtype', rowtype)

    rows = np.loadtxt(name + '_rows.csv', dtype=np.string_, delimiter=';', skiprows=1, usecols=(1,))
    default_row_strat, default_row_strat_name = load_stratification(rows, rowtype, name.split('/')[-1])

    if os.path.exists(name + '_desc.csv'):  # table case
      h5.create_array(group, 'rows', rows)
      h5.set_node_attr(group, 'type', 'table')
      import csv
      with open(name + '_desc.csv', 'r') as cc:
        desc = dict()
        lookup = dict(uint8=tables.UInt8Col, uint16=tables.UInt16Col, uint32=tables.UInt32Col,
                      int8=tables.Int8Col, int16=tables.Int16Col, int32=tables.Int32Col,
                      float16=tables.Float16Col, float32=tables.Float32Col, float64=tables.Float64Col,
                      bool=tables.BoolCol)
        columns = []
        mapper = []
        for i, row in enumerate(csv.reader(cc, delimiter=';')):
          if i == 0:
            continue
          t = None
          pos = int(row[0])
          column = dict(key=clean_name(row[1]), name=row[1])
          if row[2] == 'string':
            t = tables.StringCol(int(row[3]), pos=pos)
            column['type'] = 'string'
            m = str
          elif row[2] == 'categorical':
            keys = row[3:]
            if 'NA' not in keys:
              keys.append('NA')
            print(keys)
            enum_ = tables.misc.enum.Enum(keys)
            column['type'] = 'categorical'
            column['categories'] = keys
            if 'deceased' in keys:
              column['colors'] = ['#e41a1b', '#377eb8', '#4c4c4c']
              column['names'] = ['Deceased', 'Living', 'NA']
            if 'male' in keys:
              column['colors'] = ['blue', 'red', '#4c4c4c']
              column['names'] = ['Male', 'Female', 'NA']
            t = tables.EnumCol(enum_, 'NA', base='uint8', pos=pos)

            def wrap(e):  # wrap in a function for the right scope
              return lambda x: e['deceased' if x == 'dead' else ('living' if x == 'alive' else x)]

            m = wrap(enum_)
          else:
            t2 = row[2]
            t = lookup[row[2]](pos=pos)
            if t2.startswith('float'):

              def to_float(x):
                return np.NaN if x == 'NA' or x == '' else float(x)

              m = to_float
              column['type'] = 'real'
            else:
              missing = np.iinfo(getattr(np, row[2])).min
              print((row[2], missing))

              def to_int(x):
                return missing if x == 'NA' or x == '' else int(x)

              m = to_int
              column['type'] = 'int'
              column['missing'] = missing
            column['range'] = [None, None]
          desc[clean_name(row[1])] = t
          columns.append(column)
          mapper.append(m)

      table = h5.create_table(group, 'table', desc)
      with open(name + '_data.csv', 'r') as d:
        entry = table.row
        for row in csv.reader(d, delimiter=';'):
          for col, m, v in zip(columns, mapper, row):
            v = m(v)
            entry[col['key']] = v
            if col['type'] == 'real' or col['type'] == 'int':
              if col['type'] == 'int' and col['missing'] == v:  # exclude missing value from range computation
                v = None
              old = col['range'][0]
              col['range'][0] = v if v is not None and (old is None or v < old) else old
              old = col['range'][1]
              col['range'][1] = v if v is not None and (old is None or v > old) else old
          entry.append()

      h5.set_node_attr(group, 'columns', columns)

    elif os.path.exists(name + '_cols.csv'):  # matrix case
      h5.set_node_attr(group, 'type', 'matrix')

      with open(name + '_cols.csv', 'r') as cc:
        line = cc.readline().split(';')
        coltype = line[1].strip()
        h5.set_node_attr(group, 'coltype', coltype)

        mtype = [m.strip() for m in cc.readline().split(';')[2:]]

      cols = np.loadtxt(name + '_cols.csv', dtype=np.string_, delimiter=';', skiprows=1, usecols=(1,))
      default_col_strat, default_col_strat_name = load_stratification(cols, coltype, name.split('/')[-1])
      print(mtype)
      print((mtype[0]))

      if mtype[0] == 'float32':
        print('float32')

        with open(name + '_desc.json') as fs:
          stats = json.load(fs)
        h5.set_node_attr(group, 'value', 'real')
        with open(name + '_raw.csv') as fs:
          data = np.genfromtxt(fs, dtype=np.float32, delimiter='\t', missing_values='NaN', filling_values=np.NaN)
          # data = data[...,0:data.shape[1]]
          if stats['transposed']:
            data = np.transpose(data)
            # print
        min_v = stats.get('min', np.nanmin(data))
        max_v = stats.get('max', np.nanmax(data))
        data_raw = data
        data = data_raw.clip(min_v, max_v)
        h5.set_node_attr(group, 'range', [min_v, max_v])
        h5.set_node_attr(group, 'center', stats.get('center', 0))
      elif mtype[0] == 'int32':
        print('int32')
        h5.set_node_attr(group, 'value', 'int')
        missing = np.iinfo(np.int32).min
        h5.set_node_attr(group, 'missing', missing)
        data = np.genfromtxt(f, dtype=np.int32, delimiter=';', missing_values='NaN', filling_values=missing)
        data = data[..., 0:data.shape[1] - 1]
        h5.set_node_attr(group, 'range', [np.nanmin(data), np.nanmax(data)])
      elif mtype[0] == 'categorical':
        keys = mtype[1:]
        if '-2147483648' in keys:
          keys.remove('-2147483648')
        if 'UNKN@WN' in keys:
          keys.remove('UNKN@WN')
        keys = sorted(map(int, keys))
        keys.append(-128)
        print(keys)
        h5.set_node_attr(group, 'value', 'categorical')
        h5.set_node_attr(group, 'categories', keys)
        if -2 in keys:
          # 0;-1;-2;2;1
          h5.set_node_attr(group, 'colors', ['#0571b0', '#92c5de', '#dcdcdc', '#eeb3bb', '#ca0020', '#4c4c4c'])
          h5.set_node_attr(group, 'names',
                           ['Homozygous deletion', 'Heterozygous deletion', 'NORMAL', 'Low level amplification',
                            'High level amplification', 'Unknown'])
        elif 1 in keys:
          h5.set_node_attr(group, 'colors', ['#dcdcdc', '#ff0000', '#4c4c4c'])
          h5.set_node_attr(group, 'names', ['Not Mutated', 'Mutated', 'Unknown'])
        data = np.genfromtxt(f, dtype=np.int8, delimiter=';', missing_values={'-2147483648', 'UNKN@WN', 'NA', ''},
                             filling_values=-128)
        data = data[..., 0:data.shape[1] - 1]

      if coltype == 'TCGA_SAMPLE':  # transpose
        data = np.transpose(data)
        coltype, rowtype = rowtype, coltype
        rows, cols = cols, rows

        default_row_strat, default_col_strat = default_col_strat, default_row_strat
        default_row_strat_name, default_col_strat_name = default_col_strat_name, default_row_strat_name

      if default_col_strat and len(default_col_strat) == len(cols):
        print(('apply column stratification %s', default_col_strat_name))
        cols = cols[default_col_strat]
        data = data[:, default_col_strat]

      h5.set_node_attr(group, 'rowtype', rowtype)
      h5.set_node_attr(group, 'coltype', coltype)
      h5.create_array(group, 'rows', rows)
      h5.create_array(group, 'cols', cols)
      h5.create_array(group, 'data', data)

    h5.flush()

  h5.close()


for f in glob.glob('/vagrant/_data/calexport2hdf/*'):
  if os.path.isdir(f):
    convert_it(f)



###############################################################################
# Caleydo - Visualization for Molecular Biology - http://caleydo.org
# Copyright (c) The Caleydo Team. All rights reserved.
# Licensed under the new BSD license, available at http://caleydo.org/license
###############################################################################


def phovea(registry):
  """
  register extension points
  :param registry:
  """
  # generator-phovea:begin
  registry.append('dataset-provider', 'dataset-hdf', 'phovea_data_hdf.hdf', {})

  registry.append('json-encoder', 'dataset-hdf', 'phovea_data_hdf.json_encoder', {})

  registry.append('json-encoder', 'bytes-to-string-encoder', 'phovea_data_hdf.bytes_to_string_encoder', {})
  # generator-phovea:end
  pass


def phovea_config():
  """
  :return: file pointer to config file
  """
  from os import path
  here = path.abspath(path.dirname(__file__))
  config_file = path.join(here, 'config.json')
  return config_file if path.exists(config_file) else None




import os
import numpy as np
import tables
import phovea_server.range as ranges
import itertools
from phovea_server.dataset_def import ADataSetProvider, AColumn, AMatrix, AStratification, ATable, AVector

__author__ = 'sam'


def assign_ids(ids, idtype):
  import phovea_server.plugin

  manager = phovea_server.plugin.lookup('idmanager')
  return np.array(manager(ids, idtype))


def _resolve_categories(attrs):
  cats = attrs['categories']
  if isinstance(cats[0], str) or isinstance(cats[0], str):  # categories are strings
    converter = tables.misc.enum.Enum(cats)
    # create a numpy function out of it
    converter = np.vectorize(converter, otypes=['S' + str(max((len(c) for c in cats)))])
  else:
    converter = None
  if 'names' in attrs:
    names = attrs['names']
    colors = attrs['colors']
    if len(names) == len(cats) - 1:  # unknown missing
      names.insert(0, 'UNKN@WN')
      colors.insert(0, '#4c4c4c')
    cats = [dict(name=cat, label=name, color=col) for cat, name, col in
            zip(cats, names, colors)]

  return cats, converter


class HDFMatrix(AMatrix):
  def __init__(self, group, project):
    super(HDFMatrix, self).__init__(group._v_title, project, group._v_attrs.type.decode('utf-8'))
    self._group = group
    self._project = project
    self.path = self._group._v_pathname
    self._rowids = None
    self._colids = None
    self._range = None
    self.rowtype = self._group._v_attrs.rowtype.decode('utf-8')
    self.coltype = self._group._v_attrs.coltype.decode('utf-8')
    self.value = self._group._v_attrs.value.decode('utf-8')
    self.shape = self._group.data.shape
    if self.value == 'categorical':
      self.categories, self._converter = _resolve_categories(group._v_attrs)

  @property
  def range(self):
    if 'range' in self._group._v_attrs:
      return self._group._v_attrs['range']
    if self._range is not None:
      return self._range
    d = self._group.data
    self._range = [np.nanmin(d), np.nanmax(d)]
    return self._range

  def idtypes(self):
    return [self.rowtype, self.coltype]

  def to_description(self):
    r = super(HDFMatrix, self).to_description()
    r['rowtype'] = self.rowtype
    r['coltype'] = self.coltype
    r['value'] = v = dict(type=self.value)
    if self.value == 'real' or self.value == 'int':
      v['range'] = self.range
    if self.value == 'int' and hasattr(self._group._v_attrs, 'missing'):
      v['missing'] = self._group._v_attrs.missing.decode('utf-8')
    elif self.value == 'categorical':
      v['categories'] = self.categories

    if 'center' in self._group._v_attrs:
      v['center'] = self._group._v_attrs['center']
    r['size'] = self.shape
    return r

  def mask(self, arr):
    if self.value == 'int' and hasattr(self._group._v_attrs, 'missing'):
      missing = self._group._v_attrs.missing.decode('utf-8')
      import numpy.ma as ma
      return ma.masked_equal(arr, missing)

    if self.value == 'categorical' and self._converter is not None:
      return np.array(self._converter(arr))
    return np.array(arr)

  def asnumpy(self, range=None):
    n = self._group.data
    if range is None:
      return self.mask(n)
    rows = range[0].asslice()
    cols = range[1].asslice()
    d = None
    if isinstance(rows, list) and isinstance(cols, list):
      # fancy indexing in two dimension doesn't work
      d_help = n[rows, :]
      d = d_help[:, cols]
    else:
      d = n[rows, cols]

    if d.ndim == 1:
      # two options one row and n columns or the other way around
      if rows is Ellipsis or (isinstance(rows, list) and len(rows) > 1):
        d = d.reshape((d.shape[0], 1))
      else:
        d = d.reshape((1, d.shape[0]))
    elif d.ndim == 0:
      d = d.reshape((1, 1))
    return self.mask(d)

  def rows(self, range=None):
    n = np.array(self._group.rows)
    if range is None:
      return n
    return n[range.asslice()]

  def rowids(self, range=None):
    if self._rowids is None:
      self._rowids = assign_ids(self.rows(), self.rowtype)
    n = self._rowids
    if range is None:
      return n
    return n[range.asslice()]

  def cols(self, range=None):
    n = np.array(self._group.cols)
    if range is None:
      return n
    return n[range.asslice()]

  def colids(self, range=None):
    if self._colids is None:
      self._colids = assign_ids(self.cols(), self.coltype)
    n = self._colids
    if range is None:
      return n
    return n[range.asslice()]


class HDFVector(AVector):
  def __init__(self, group, project):
    super(HDFVector, self).__init__(group._v_title, project, group._v_attrs.type.decode('utf-8'))
    self._group = group
    self._project = project
    self._rowids = None
    self._range = None
    self.idtype = self._group._v_attrs.rowtype.decode('utf-8')
    self.value = self._group._v_attrs.value.decode('utf-8')
    self.shape = len(self._group.data)

  @property
  def range(self):
    if 'range' in self._group._v_attrs:
      return self._group._v_attrs['range']
    if self._range is not None:
      return self._range
    d = self._group.data
    self._range = [np.nanmin(d), np.nanmax(d)]
    return self._range

  def idtypes(self):
    return [self.idtype]

  def to_description(self):
    r = super(HDFVector, self).to_description()
    r['idtype'] = self.idtype
    r['value'] = dict(type=self.value, range=self.range)
    if self.value == 'int' and hasattr(self._group._v_attrs, 'missing'):
      r['value']['missing'] = self._group._v_attrs.missing.decode('utf-8')
    if 'center' in self._group._v_attrs:
      r['value']['center'] = self._group._v_attrs['center']
    r['size'] = [self.shape]
    return r

  def mask(self, arr):
    if self.value == 'int' and hasattr(self._group._v_attrs, 'missing'):
      missing = self._group._v_attrs.missing.decode('utf-8')
      import numpy.ma as ma
      return ma.masked_equal(arr, missing)
    return arr

  def asnumpy(self, range=None):
    n = self._group.data
    if range is None:
      return self.mask(n)
    d = n[range[0].asslice()]
    if d.ndim == 0:
      d = d.reshape((1,))
    return self.mask(d)

  def rows(self, range=None):
    n = np.array(self._group.rows)
    if range is None:
      return n
    return n[range.asslice()]

  def rowids(self, range=None):
    if self._rowids is None:
      self._rowids = assign_ids(self.rows(), self.rowtype)
    n = self._rowids
    if range is None:
      return n
    return n[range.asslice()]


class HDFGroup(object):
  def __init__(self, name, offset, data, color):
    self.name = name
    self.data = data
    self.range = ranges.from_slice(offset, offset + len(data))
    self.color = color

  def __len__(self):
    return len(self.data)

  def dump(self):
    return dict(name=self.name, range=str(self.range), color=self.color)

  def dump_desc(self):
    return dict(name=self.name, size=len(self), color=self.color)


def guess_color(name, i):
  name = name.lower()
  colors = dict(male='blue', female='red', deceased='#e41a1b', living='#377eb8')
  if name in colors:
    return colors[name]
  colors = ['#8dd3c7', '#ffffb3', '#bebada', '#fb8072', '#80b1d3', '#fdb462', '#b3de69', '#fccde5', '#d9d9d9', '#bc80bd', '#ccebc5', '#ffed6f']
  return colors[i % len(colors)]


class HDFStratification(AStratification):
  def __init__(self, group, project):
    super(HDFStratification, self).__init__(group._v_title, project, group._v_attrs.type.decode('utf-8'))
    self._group = group
    self._project = project
    self._rowids = None
    self.idtype = self._group._v_attrs.idtype.decode('utf-8')
    self._groups = None

  def idtypes(self):
    return [self.idtype]

  def to_description(self):
    r = super(HDFStratification, self).to_description()
    r['idtype'] = self.idtype
    if 'origin' in self._group._v_attrs:
      r['origin'] = self._project + '/' + self._group._v_attrs.origin.decode('utf-8')
    r['groups'] = [g.dump_desc() for g in self.groups()]
    r['ngroups'] = len(r['groups'])
    r['size'] = [sum((g['size'] for g in r['groups']))]
    return r

  def _rows(self):
    return np.concatenate([g.data for g in self.groups()])

  def rows(self, range=None):
    n = self._rows()
    if range is None:
      return n
    return n[range[0].asslice()]

  def rowids(self, range=None):
    if self._rowids is None:
      self._rowids = assign_ids(self.rows(), self.idtype)
    n = self._rowids
    if range is None:
      return n
    return n[range.asslice()]

  def groups(self):
    if self._groups is None:
      self._groups = []
      i = 0
      values = iter(self._group._v_children.values())
      for j, g in enumerate(sorted(values, key=lambda x: x._v_title)):
        name = g._v_title
        color = g._v_attrs['color'] if 'color' in g._v_attrs else guess_color(name, j)
        length = len(g)
        self._groups.append(HDFGroup(name, i, g, color))
        i += length
    return self._groups

  def __getitem__(self, item):
    group = getattr(self._group, item)
    return group

  def asjson(self, range=None):
    r = dict(rows=self.rows(range), rowIds=self.rowids(range), groups=[g.dump() for g in self.groups()])
    return r


class HDFColumn(AColumn):
  def __init__(self, attrs, group):
    super(HDFColumn, self).__init__(attrs['name'], attrs['type'])
    self._group = group
    self.key = attrs['key']
    if self.type == 'categorical':
      self.categories, self._converter = _resolve_categories(attrs)
    elif self.type == 'int' or self.type == 'real':
      self.range = attrs['range'] if 'range' in attrs else self.compute_range()
      self.missing = attrs.get('missing', None)

  def compute_range(self):
    d = self._group.table.col(self.key)
    return [np.nanmin(d), np.nanmax(d)]

  def convert_category(self, vs):
    if self._converter is not None:
      return self._converter(vs)
    return vs

  def mask(self, arr):
    if self.type == 'int' and self.missing is not None:
      import numpy.ma as ma
      return ma.masked_equal(arr, self.missing)

    if self.type == 'categorical' and self._converter is not None:  # convert categorical columns
      return self._converter(arr)
    return arr

  def asnumpy(self, range=None):
    n = self._group.table.col(self.key)
    if range is not None:
      n = n[range[0].asslice()]
    return self.mask(n)

  def dump(self):
    value = dict(type=self.type)
    if self.type == 'categorical':
      value['categories'] = self.categories
    if self.type == 'int' or self.type == 'real':
      value['range'] = self.range
    if self.type == 'int' and self.missing is not None:
      value['missing'] = self.missing
    return dict(name=self.name, value=value, column=self.key)


class HDFTable(ATable):
  def __init__(self, group, project):
    super(HDFTable, self).__init__(group._v_title, project, group._v_attrs.type.decode('utf-8'))
    self._group = group
    self._project = project

    self.columns = [HDFColumn(a, group) for a in group._v_attrs.columns]
    self._rowids = None
    self.idtype = self._group._v_attrs.rowtype.decode('utf-8')

  def idtypes(self):
    return [self.idtype]

  def to_description(self):
    r = super(HDFTable, self).to_description()
    r['idtype'] = self.idtype
    r['columns'] = [d.dump() for d in self.columns]
    r['size'] = [len(self._group.table), len(self.columns)]
    return r

  def rows(self, range=None):
    n = np.array(self._group.rows)
    if range is None:
      return n
    return n[range.asslice()]

  def rowids(self, range=None):
    if self._rowids is None:
      self._rowids = assign_ids(self.rows(), self.idtype)
    n = self._rowids
    if range is None:
      return n
    return n[range.asslice()]

  def aspandas(self, range=None):
    import pandas as pd
    n = pd.DataFrame.from_records(self._group.table[:])
    # ensure right column order
    n = n[[item.key for item in self.columns]]

    # convert categorical enums
    # rename variable to avoid shadowing
    for item in self.columns:
      if item.type == 'categorical':
        n[item.key] = item.convert_category(n[item.key])

    if range is None:
      return n
    return n.iloc[range[0].asslice(no_ellipsis=True)]

  def filter(self, query):
    # perform the query on rows and cols and return a range with just the matching one
    # np.argwhere
    return ranges.all()


class HDFProject(object):
  def __init__(self, filename, base_dir):
    self.filename = filename
    p = os.path.relpath(filename, base_dir)
    project, _ = os.path.splitext(p)
    project = project.replace('.', '_')
    self._h = tables.open_file(filename, 'r')

    self.entries = []
    for group in self._h.walk_groups('/'):
      if 'type' not in group._v_attrs:
        continue
      t = group._v_attrs.type.decode('utf-8')
      if t == 'matrix':
        self.entries.append(HDFMatrix(group, project))
      elif t == 'stratification':
        self.entries.append(HDFStratification(group, project))
      elif t == 'table':
        self.entries.append(HDFTable(group, project))
      elif t == 'vector':
        self.entries.append(HDFVector(group, project))

  def __iter__(self):
    return iter(self.entries)

  def __len__(self):
    return len(self.entries)

  def __getitem__(self, dataset_id):
    for f in self.entries:
      if f.id == dataset_id:
        return f
    return None


class HDFFilesProvider(ADataSetProvider):
  def __init__(self):
    from phovea_server import config
    # check initialization
    if config._c is None:
      config._initialize()
    conf = config.view('phovea_data_hdf')
    from phovea_server.util import glob_recursivly
    base_dir = config.get('dataDir', 'phovea_server')
    self.files = [HDFProject(f, base_dir) for f in glob_recursivly(base_dir, conf.get('glob'))]

  def __len__(self):
    return sum((len(f) for f in self))

  def __iter__(self):
    return iter((f for f in itertools.chain(*self.files) if f.can_read()))

  def __getitem__(self, dataset_id):
    for f in self.files:
      r = f[dataset_id]
      if r is not None:
        return r
    return None


if __name__ == '__main__':
  # app.debug1 = True

  c = HDFFilesProvider()


def create():
  return HDFFilesProvider()



import tables


__author__ = 'Samuel Gratzl'


class TablesEncoder(object):
  def __contains__(self, obj):
    if isinstance(obj, tables.Array):
      return True
    return False

  def __call__(self, obj, base_encoder):
    if isinstance(obj, tables.Array):
      if obj.ndim == 1:
        return [x for x in obj]
      else:
        return [base_encoder.default(obj[i]) for i in range(obj.shape[0])]
    return None


n = TablesEncoder()


def create():
  return n



###############################################################################
# Caleydo - Visualization for Molecular Biology - http://caleydo.org
# Copyright (c) The Caleydo Team. All rights reserved.
# Licensed under the new BSD license, available at http://caleydo.org/license
###############################################################################

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))


def read_it(name):
  with open(path.join(here, name), encoding='utf-8') as f:
    return f.read()


# read package.json information
with open(path.join(here, 'package.json'), encoding='utf-8') as json_data:
  import json

  pkg = json.load(json_data)


def packaged(*files):
  r = {}
  global pkg
  r[pkg['name']] = list(files)
  return r


def requirements(file):
  return [r.strip() for r in read_it(file).strip().split('\n') if not r.startswith('-e git+https://')]


def to_version(v):
  import datetime
  now = datetime.datetime.utcnow()
  return v.replace('SNAPSHOT', now.strftime('%Y%m%d-%H%M%S'))


setup(
  name=pkg['name'].lower(),
  version=to_version(pkg['version']),
  url=pkg['homepage'],
  description=pkg['description'],
  long_description=read_it('README.md'),
  long_description_content_type='text/markdown',
  keywords=pkg.get('keywords', ''),
  author=pkg['author']['name'],
  author_email=pkg['author']['email'],
  license=pkg['license'],
  zip_safe=False,

  entry_points={
    'phovea.registry': ['{0} = {0}:phovea'.format(pkg['name'])],
    'phovea.config': ['{0} = {0}:phovea_config'.format(pkg['name'])]
  },

  # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
  classifiers=[
    'Intended Audience :: Developers',
    'Operating System :: OS Independent',
    # Pick your license as you wish (should match "license" above)
    'License :: OSI Approved :: ' + ('BSD License' if pkg['license'] == 'BSD-3-Clause' else pkg['license']),
    'Programming Language :: Python',
    'Programming Language :: Python :: 3.7'
  ],

  # You can just specify the packages manually here if your project is
  # simple. Or you can use find_packages().
  packages=find_packages(exclude=['docs', 'tests*']),

  # List run-time dependencies here.  These will be installed by pip when
  # your project is installed. For an analysis of "install_requires" vs pip's
  # requirements files see:
  # https://packaging.python.org/en/latest/requirements.html
  install_requires=requirements('requirements.txt'),
  tests_require=requirements('requirements_dev.txt'),

  # If there are data files included in your packages that need to be
  # installed, specify them here.  If using Python 2.6 or less, then these
  # have to be included in MANIFEST.in as well.
  package_data=packaged('config.json', 'buildInfo.json'),

  # Although 'package_data' is the preferred approach, in some case you may
  # need to place data files outside of your packages. See:
  # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
  # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
  data_files=[]  # [('my_data', ['data/data_file'])],
)



"""
The HDF files in the _data directory contain objects of type 'bytes'. In Python 3.X the unicode type has been renamed as str and the older str type has been replaced by bytes.
This encoder is required to handle this change for the update to Python 3.7 by decoding bytes objects to strings that can be read.
"""


class BytesToStringEncoder(object):
  def __contains__(self, obj):
    if isinstance(obj, bytes):
      return True
    return False

  def __call__(self, obj, base_encoder):
    if isinstance(obj, bytes):
      return obj.decode('utf-8')
    return None


encoder = BytesToStringEncoder()


def create():
  return encoder
